Package: ScaleSpikeSlab
Type: Package
Title: Scalable Spike-and-Slab
Version: 1.0
Date: 2022-05-13
Authors@R: 
  c(person(given = "Niloy",
           family = "Biswas",
           role = c("aut", "cre"),
           email = "niloy_biswas@g.harvard.edu",
           comment = c(ORCID = "0000-0001-9081-5702")),
    person(given = "Lester",
           family = "Mackey",
           role = "aut"),
    person(given = "Xiao-Li",
           family = "Meng",
           role = "aut"))
Description: A scalable Gibbs sampling implementation for high dimensional Bayesian regression with the continuous spike-and-slab prior. Niloy Biswas, Lester Mackey and Xiao-Li Meng, "Scalable Spike-and-Slab" (2022) <arXiv:2204.01668>.
License: GPL (>= 2)
Imports: Rcpp, stats, TruncatedNormal
LinkingTo: Rcpp, RcppEigen
RoxygenNote: 7.1.2
NeedsCompilation: yes
Packaged: 2022-05-17 20:40:23 UTC; niloybiswas
Author: Niloy Biswas [aut, cre] (<https://orcid.org/0000-0001-9081-5702>),
  Lester Mackey [aut],
  Xiao-Li Meng [aut]
Maintainer: Niloy Biswas <niloy_biswas@g.harvard.edu>
Depends: R (>= 3.5.0)
Repository: CRAN
Date/Publication: 2022-05-18 17:00:07 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 12:33:07 UTC; unix
Archs: ScaleSpikeSlab.so.dSYM
